@@ -7,12 +7,6 @@ function(X, r = NULL, ReferenceType, NeighborType = ReferenceType, Weighted = FA
     CheckdbmssArguments()
   }
   
-  # Select the bandwith: original choice by Duranton and Overman or optimized one.
-  if (Original) {
-    bw <- "nrd0"
-  } else {
-    bw <- "sj"
-  }
   # The default number of values used by density is 512. It may be changed if needed.
   n <- 512
   
@@ -67,10 +61,10 @@ function(X, r = NULL, ReferenceType, NeighborType = ReferenceType, Weighted = FA
     # Round distances to save memory
     # Prepare steps so that 1024*Approximate steps are between 0 and rmax.
     # Pairs further than 2*rmax apart will be stored in an extra element.
-    rseq <- seq(from = 0, to = rmax*2, length.out = 2048*Approximate)
+    rseq <- seq(from=0, to=rmax*2, length.out=2048*Approximate)
     # Number of distances
     Nr <- length(rseq)
-    # Prepare a matrix, single line, one value for each distance + 1 extra for pairs far away.
+    # Prepare a matrix: single line, one value for each distance + 1 extra for pairs far away.
     NeighborWeight <- matrix(0.0, nrow=1, ncol=Nr+1)
     # Weights
     if (Weighted) {
@@ -82,34 +76,35 @@ function(X, r = NULL, ReferenceType, NeighborType = ReferenceType, Weighted = FA
     # Call C routine to fill NeighborWeights
     CountNbdKd(rseq, Y$x, Y$y, Weight, NeighborWeight, IsReferenceType, IsNeighborType)
     
-    # Adjust distances: values are the centers of intervals
+    # Adjust distances: values are the centers of intervals.
+    # rseq becomes a vector (it was a 1-row matrix)
     rseq <- c(0, (rseq[2:Nr]+rseq[1:Nr-1])/2)
-
-    # Estimate the density. Change the bandwith according to adjust if requested.
+    
+    # Estimate the bandwith according to adjust if requested.
+    # Distances are the values of rseq corresponding to at least a pair of points i.e. NeighborWeight > 0
+    # Ignore the last value of NeighborWeight which contains far neighbors
     if (Original) {
-      bw <- stats::bw.nrd0(rseq) * Adjust
+      bw <- stats::bw.nrd0(rseq[NeighborWeight[-length(NeighborWeight)]>0]) * Adjust
     } else {
-      bw <- stats::bw.SJ(rseq) * Adjust
+      bw <- stats::bw.SJ(rseq[NeighborWeight[-length(NeighborWeight)]>0]) * Adjust
     }
+    
+    # Add a last value to rseq equal to 2*rmax+4bw (i.e. ignored by the estimation of density at rmax) for far neighbors
+    rseq <- c(rseq, rmax*2 + 4*bw)
+    # Estimated density is false above 2rmax-4bw, but it will be censored at rmax.
+    # The total mass is correct. It is needed for normalization after mirroring.
 
-    # The last element of the vector NeighborWeight contains the weight of pairs farther than 2 rmax
-    FarWeight <- NeighborWeight[Nr+1]
-    # Keep the other elements
-    NeighborWeight <- NeighborWeight[-(Nr+1)]
-    # Total density is not 1 because far weights will be dropped
-    SumNeighborWeight <- sum(NeighborWeight)
-    TotalDensity <- SumNeighborWeight/(SumNeighborWeight+FarWeight)
     # Prepare reflection. Distances below 4bw are mirrored around 0. The first one is 0: ignore it. Code adapted from GoFKernel::density.reflected
     Reflected <- which(rseq <= 4*bw)[-1]
     rseq <- c(rseq, -rseq[Reflected])
     NeighborWeight <- c(NeighborWeight, NeighborWeight[Reflected])
-    # Update the sum
+    # Sum of weights to normalize them to avoid warning during density estimation
     SumNeighborWeight <- sum(NeighborWeight)
     # Estimate density and the density of the mirrored values (below 0) to renormalize later
     Density <- stats::density(rseq, weight=NeighborWeight/SumNeighborWeight, from=0, to=rmax, bw=bw, n=n)
     Mirrored <- stats::density(rseq, weight=NeighborWeight/SumNeighborWeight, to=0, bw=bw)
     # Renormalize density because mirrored distances decreased it
-    Density$y <- Density$y / (TotalDensity - mean(Mirrored$y)*diff(range(Mirrored$x)))
+    Density$y <- Density$y / (1 - mean(Mirrored$y)*diff(range(Mirrored$x)))
     
   } else {
     # Classical estimation of distances
@@ -156,10 +151,11 @@ function(X, r = NULL, ReferenceType, NeighborType = ReferenceType, Weighted = FA
     }
     
     # Estimate the density. Change the bandwith according to adjust if requested.
+    # Only pairs of points up to 2rmax are considered for consistency with approximated computation
     if (Original) {
-      bw <- stats::bw.nrd0(Dist) * Adjust
+      bw <- stats::bw.nrd0(Dist[Dist<=rmax*2]) * Adjust
     } else {
-      bw <- stats::bw.SJ(Dist) * Adjust
+      bw <- stats::bw.SJ(Dist[Dist<=rmax*2]) * Adjust
     }
     # Prepare reflection. Distances below rmin + 4bw are mirrored. Code adapted from GoFKernel::density.reflected
     Reflected <- which(Dist <= rmin + 4*bw)
